import { ILayer, ILngLat, ITile } from '@antv/l7-core';
import { SourceTile } from '@antv/l7-utils';
export default abstract class Tile implements ITile {
    x: number;
    y: number;
    z: number;
    key: string;
    protected parent: ILayer;
    sourceTile: SourceTile;
    visible: boolean;
    protected layers: ILayer[];
    isLoaded: boolean;
    constructor(sourceTile: SourceTile, parent: ILayer);
    getLayers(): ILayer[];
    styleUpdate(...arg: any): void;
    abstract initTileLayer(): Promise<void>;
    lnglatInBounds(lnglat: ILngLat): boolean;
    protected addMask(layer: ILayer, mask: ILayer): Promise<void>;
    protected addLayer(layer: ILayer): Promise<void>;
    updateVisible(value: boolean): void;
    updateOptions(key: string, value: any): void;
    /**
     * 一个 Tile 可能有多个 layer，但是在发生拾取、点击事件的时候只有一个生效
     */
    getMainLayer(): ILayer | undefined;
    getFeatures(sourceLayer: string | undefined): any[];
    /**
     * 在一个 Tile 中可能存在一个相同 ID 的 feature
     * @param id
     * @returns
     */
    getFeatureById(id: number): any[];
    destroy(): void;
}
